'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _amqplib = require('amqplib');

var _amqplib2 = _interopRequireDefault(_amqplib);

var _chance = require('chance');

var _chance2 = _interopRequireDefault(_chance);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _events = require('events');

var _logger = require('./logger');

var _logger2 = _interopRequireDefault(_logger);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; } /**
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * Created by asafdavid on 07/12/2015.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                */
// External packages


var RECONNECTION_TIMEOUT = 1000;
var RETRY_CONNECT_TIMEOUT = 5000;
var chance = new _chance2.default();

/**
 * Represent a connection to the AMQP server
 */

var Connection = function (_EventEmitter) {
  _inherits(Connection, _EventEmitter);

  /**
   * Creates a new connection object
   * @param connectionString
   */
  function Connection(connectionString) {
    _classCallCheck(this, Connection);

    var _this = _possibleConstructorReturn(this, (Connection.__proto__ || Object.getPrototypeOf(Connection)).call(this));

    _this.connectionString = _lodash2.default.castArray(connectionString || 'amqp://localhost');
    return _this;
  }

  /**
   * Returns a random connection string from the pool
   * @returns {*}
   */


  _createClass(Connection, [{
    key: 'getConnectionString',
    value: function getConnectionString() {
      return chance.pickone(this.connectionString);
    }

    /**
     * Connects to the wanted AMQP server and tries to maintain the connection.
     */

  }, {
    key: 'connect',
    value: function connect() {
      var _this2 = this;

      var connectionString = this.getConnectionString();
      _logger2.default.instance.info('[AMQP] picked a connection string ' + connectionString);
      return _amqplib2.default.connect(connectionString).then(function (conn) {
        _this2.conn = conn;

        _logger2.default.instance.info('[AMQP] connected to ' + connectionString);

        // Opens a channel
        conn.createConfirmChannel().then(function (ch) {
          _this2.channel = ch;
          _this2.emit('open', conn);

          _this2.channel.on('error', function (err) {
            _logger2.default.instance.error('[AMQP] error occurred in channel', { error: err });
            setTimeout(function () {
              return _this2.connect();
            }, RECONNECTION_TIMEOUT);
          });
        });

        // Handle connection error
        conn.on('error', function (err) {
          if (err.message === 'read ECONNRESET') {
            _logger2.default.instance.error('[AMQP] reconnecting');
            setTimeout(function () {
              return _this2.connect();
            }, RECONNECTION_TIMEOUT);
          } else {
            _logger2.default.instance.error('[AMQP] conn error', { error: err });
            _this2.emit('error', err);
          }
        });

        conn.on('close', function (msg) {
          if (!!msg && msg !== 'Closed by client' && !msg.match(/Error/)) {
            _logger2.default.instance.error('[AMQP] reconnecting');
            setTimeout(function () {
              return _this2.connect();
            }, RECONNECTION_TIMEOUT);
          } else {
            _this2.emit('close');
          }
        });
      }, function (err) {
        _logger2.default.instance.error('[AMQP] conn error', { error: err });
        if (err.message.match(/ECONNREFUSED|ENOTFOUND/)) {
          _logger2.default.instance.error('[AMQP] reconnecting');
          setTimeout(function () {
            return _this2.connect();
          }, RETRY_CONNECT_TIMEOUT);
        } else {
          _this2.emit('error', err);
        }
      });
    }
  }, {
    key: 'close',
    value: function close() {
      var _this3 = this;

      _logger2.default.instance.info('[AMQP] Closing channel and connection');
      return this.channel.waitForConfirms().then(function () {
        return _this3.channel.close();
      }).then(function () {
        return _this3.conn.close();
      });
    }
  }]);

  return Connection;
}(_events.EventEmitter);

exports.default = Connection;