'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = composePromise;

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Composes single-argument functions that return promises from right to left.
 * Heavily inspired by Redux.
 *
 * @param {...Function} funcs The functions to compose.
 * @returns {Function} A function obtained by composing the argument functions
 * from right to left. For example, compose(f, g, h) is identical to doing
 * (...args) => h(...args).then(...args => f()).then(...args => g(...args)).
 */

function composePromise() {
  for (var _len = arguments.length, funcs = Array(_len), _key = 0; _key < _len; _key++) {
    funcs[_key] = arguments[_key];
  }

  // Return a method that returns the message
  if (funcs.length === 0) {
    return function (arg) {
      return _bluebird2.default.resolve(arg);
    };
  }

  // One middleware - return as is
  if (funcs.length === 1) {
    return funcs[0];
  }

  // Compose middleware
  return funcs.reduce(function (a, b) {
    return function () {
      return b.apply(undefined, arguments).then(function () {
        return a.apply(undefined, arguments);
      });
    };
  });
} /**
   * Created by asafdavid on 09/03/2017.
   */