'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

var _pg = require('pg');

var _moment = require('moment');

var _moment2 = _interopRequireDefault(_moment);

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _logger = require('../logger');

var _logger2 = _interopRequireDefault(_logger);

var _puller = require('./puller');

var _puller2 = _interopRequireDefault(_puller);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; } /**
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                * Created by chenrozenes and ofirhartuv on 05/11/2017.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                */


var TABLE_NAME = 'rabbit_pull_queue_sync';
var CREATE_QUERY = 'CREATE TABLE IF NOT EXISTS ' + TABLE_NAME + ' (' + 'queue CHARACTER VARYING(200) NOT NULL UNIQUE, ' + 'last_pull TIMESTAMP NOT NULL);';

var INIT_FIRST_VALUE = 'INSERT INTO ' + TABLE_NAME + ' VALUES($1,\'-infinity\') ON CONFLICT DO NOTHING';

var UPDATE_QUERY = 'UPDATE ' + TABLE_NAME + ' SET last_pull=$1 WHERE queue LIKE $2 AND last_pull<$3';

/**
 * Puller class that uses Postgres database in order to sync pulling between different workers
 */

var PostgresPuller = function (_Puller) {
  _inherits(PostgresPuller, _Puller);

  function PostgresPuller() {
    var config = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    _classCallCheck(this, PostgresPuller);

    var _this = _possibleConstructorReturn(this, (PostgresPuller.__proto__ || Object.getPrototypeOf(PostgresPuller)).call(this, config));

    _this.connStr = config.options.url;
    return _this;
  }

  _createClass(PostgresPuller, [{
    key: '_connect',
    value: function _connect() {
      var _this2 = this;

      _logger2.default.instance.info('[AMQP] PostgresPuller - connecting to Postgres: ' + this.connStr);
      this.client = new _pg.Client(this.connStr);
      return _bluebird2.default.resolve().then(function () {
        return _this2.client.connect();
      });
    }
  }, {
    key: '_disconnect',
    value: function _disconnect() {
      var _this3 = this;

      _logger2.default.instance.info('[AMQP] PostgresPuller - closing connection');
      return _bluebird2.default.resolve().then(function () {
        return _this3.client.end();
      });
    }

    /**
     * Creates the table and fills with initial values if not exist, before starting pulling loop
     * @param queue
     * @returns {*}
     * @private
     */

  }, {
    key: '_setup',
    value: function _setup(queue) {
      var _this4 = this;

      _logger2.default.instance.info('[AMQP] PostgresPuller - Creating sync table with initial values');
      return _bluebird2.default.resolve().then(function () {
        return _this4.client.query(CREATE_QUERY);
      }).then(function () {
        return _this4.client.query(INIT_FIRST_VALUE, [queue]);
      });
    }

    /**
     * Trying to update the queue row, if succeeded current worker is allowed to pull.
     *
     * @returns {*}
     */

  }, {
    key: 'shouldPull',
    value: function shouldPull() {
      var _this5 = this;

      var now = (0, _moment2.default)();
      var lessThen = (0, _moment2.default)(now).subtract(this.interval / 1000, 'seconds'); // Subtract interval

      return _bluebird2.default.resolve().then(function () {
        return _this5.client.query(UPDATE_QUERY, [now.toDate(), _this5.queue, lessThen.toDate()]);
      }).then(function (result) {
        // More than one row updated is weird
        if (result.rowCount > 1) {
          _logger2.default.instance.error('[AMQP] Puller updated ' + result.rowCount + ' row. ' + ('Syncing queue ' + _this5.queue + ' is problematic'));

          throw new Error('PostgresPuller performed bad update');
        }

        // If updated, it means the worker can pull
        return result.rowCount === 1;
      });
    }
  }, {
    key: 'start',
    value: function start(channel, queue, callback) {
      var _this6 = this;

      return this._connect().then(function () {
        return _this6._setup(queue);
      }).then(function () {
        return _get(PostgresPuller.prototype.__proto__ || Object.getPrototypeOf(PostgresPuller.prototype), 'start', _this6).call(_this6, channel, queue, callback);
      });
    }
  }, {
    key: 'stop',
    value: function stop() {
      var _this7 = this;

      return _get(PostgresPuller.prototype.__proto__ || Object.getPrototypeOf(PostgresPuller.prototype), 'stop', this).call(this).then(function () {
        return _this7._disconnect();
      });
    }
  }]);

  return PostgresPuller;
}(_puller2.default);

exports.default = PostgresPuller;