'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.NOOP = undefined;
exports.default = Middleware;

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _baseService = require('../baseService');

var _baseService2 = _interopRequireDefault(_baseService);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var NOOP = exports.NOOP = function NOOP(res) {
  return _bluebird2.default.resolve(res);
};

/**
 * Creates a Middleware decorator
 * @param pre
 * @param post
 * @constructor
 */
/**
 * Created by asafdavid on 01/03/2017.
 */
function Middleware(name) {
  var pre = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : NOOP;
  var post = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : NOOP;
  var options = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};

  if (!name) throw new TypeError('name is required');
  if (!_lodash2.default.isFunction(pre)) throw new TypeError('pre is required and must be a function');
  if (!_lodash2.default.isFunction(post)) throw new TypeError('post is required and must be a function');

  return function middlewareDecorator(target, key, descriptor) {
    // Check if a task is defined within a service
    if (!(target instanceof _baseService2.default)) {
      throw SyntaxError('@task must be defined within a service');
    }
    if (!_lodash2.default.isFunction(descriptor.value)) {
      throw SyntaxError('@Middleware(' + name + ') must be defined on a function');
    }
    // Check if it was defined on a task
    if (!target._tasks || !target._tasks[key]) {
      throw SyntaxError('@Middleware(' + name + ') must be defined on a task, use @Task before @Middleware');
    }

    // Add the middleware by order
    target._tasks[key].middleware.pre.push(pre);
    target._tasks[key].middleware.post.push(post);
    target._tasks[key].options[name] = options;
  };
}