import Promise from 'bluebird';

import { Task, Errors } from '../../src/service/decorator';
import BaseService from '../../src/service/baseService';
import BaseError from './baseError';

export class BaseClass extends BaseService {
  @Task({
    topic: 'TWILIO.SEND',
    sync: true,
  })
  sendSms() { // eslint-disable-line class-methods-use-this
    return Promise.reject('undefined task');
  }

  @Task({
    topic: 'TWILIO.SEND_WITH_BASE_ERROR',
    sync: true,
    baseError: BaseError,
  })
  sendSmsWithBaseError() { // eslint-disable-line class-methods-use-this
    return Promise.reject('undefined task');
  }

  @Task({
    topic: 'TWILIO.RECEIVE',
    sync: false,
  })
  receiveSms() { // eslint-disable-line class-methods-use-this
    return Promise.reject('undefined task');
  }
}

export class TwilioService extends BaseClass {
  constructor() {
    super('Twilio');
  }

  @Task({})
  sendSms({ number, text }) { // eslint-disable-line class-methods-use-this
    return Promise.resolve(`message sent ${text} to ${number}`);
  }

  @Task({})
  sendSmsWithBaseError({ number, text }) {
    return this.sendSms({ number, text });
  }

  @Task({})
  receiveSms() { // eslint-disable-line class-methods-use-this
    return Promise.resolve('Received a message');
  }
}

@Errors({
  baseError: BaseError,
})
class BaseClassForBaseError extends BaseService {
  @Task({
    topic: 'TWILIO.SEND',
    sync: true,
  })
  sendSms() { // eslint-disable-line class-methods-use-this
    return Promise.reject('undefined task');
  }
}

export class TwilioServiceClassErrors extends BaseClassForBaseError {
  constructor() {
    super('Twilio');
  }

  @Task({
  })
  sendSms({ number, text }) { // eslint-disable-line class-methods-use-this
    return Promise.resolve(`message sent ${text} to ${number}`);
  }
}
